@@ -640,7 +640,7 @@ class BleakClientBlueZDBus(BaseBleakClient):
                 )
                 assert_reply(reply)
                 # Simulate regular characteristics read to be consistent over all platforms.
-                value = bytearray(reply.body[0]["Percentage"].value)
+                value = bytearray([reply.body[0]["Percentage"].value])
                 logger.debug(
                     "Read Battery Level {0} | {1}: {2}".format(
                         char_specifier, self._device_path, value
